require 'spec_helper'

RSpec.describe Frozen::Template::Base do
  let(:base) {  Frozen::Template::Base.new }
  describe "#read_from_file" do
    before(:each) do
      expect(File).to receive(:read).with('test') {"test content"}
    end
    it "should return response from File.read" do
      expect(base.read_from_file('test')).to eq("test content")
    end
  end


  describe "#file_type" do
    it "should return file extension" do
      base.file_path = "test.rb"
      expect(base.file_type).to eq("rb")
    end
    it "should return last extension" do
      base.file_path = "test.html.slim"
      expect(base.file_type).to eq("slim")
    end
  end

  describe "#render_to_file" do
    it "should call File.write" do
      expect(base).to receive(:render) { "test data" }
      expect(File).to receive(:write).with("test_file.txt", "test data")
      base.render_to_file("test_file.txt")
    end
  end

  describe "#template_filename" do
    it "should return base filename" do
      base.file_path = "/test/test.html.slim"
      expect(base.template_filename).to eq("test.html.slim")
    end
  end

  describe "#build_filename" do
    it "should return filename without last extension" do
      base.file_path = "/test/test.html.slim"
      expect(base.build_filename).to eq("test.html")
    end
  end

  describe "#template_path" do
    it "should return directory portion of path" do
      base.file_path = "app/views/index.html.slim"
      expect(base.template_path).to eq("app/views")
    end
  end

  describe "#build_file_path" do
    it "should combine build_path and build_filename" do
      base.file_path = "test.html.slim"
      expect(base.build_file_path).to eq("test.html")
    end
    it "should output nested paths" do
      base.file_path = "views/something/test.html.slim"
      base.template_root_path = "views"
      expect(base.build_file_path).to eq('something/test.html')
    end
    it "should output nested paths" do
      base.file_path = "views/test.html.slim"
      base.template_root_path = "views"
      expect(base.build_file_path).to eq('test.html')
    end
    it "should ouput nested path with build_path" do
      allow(base).to receive(:build_path) { "html" }
      base.file_path = "/1/2/3/views/something/test.html.slim"
      base.template_root_path = "/1/2/3/views"
      expect(base.build_file_path).to eq('html/something/test.html')
    end
  end

  describe "#relative_path" do
    it "should return relative path based on template_root_path" do
      allow(base).to receive(:template_root_path) { "assets/stylesheets" }
      base.file_path = "assets/stylesheets/sub/sub.html.slim"
      expect(base.relative_path).to eq("sub")
    end
  end

  describe "#relative_to_root" do
    it "should get root path from subpath" do
      base = Frozen::Template::Base.new
      allow(base).to receive(:template_root_path) { "views" }
      base.file_path = "views/test.html.slim"
      expect(base.relative_to_root).to eq(".")
    end
    it "should get root path from nested subpath" do
      base = Frozen::Template::Base.new
      allow(base).to receive(:build_path)
      base.template_root_path = "views"
      base.file_path = "views/something/somewhere/test.html.slim"
      expect(base.relative_to_root).to eq("../..")
    end
  end
end
